
package one.microstream.integrations.cdi.types.extension;

/*-
 * #%L
 * microstream-integrations-cdi
 * %%
 * Copyright (C) 2019 - 2022 MicroStream Software
 * %%
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 * 
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License, v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is
 * available at https://www.gnu.org/software/classpath/license.html.
 * 
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 * #L%
 */

import java.lang.annotation.Annotation;
import java.util.Collections;
import java.util.Set;

import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.enterprise.inject.spi.InjectionPoint;
import javax.enterprise.inject.spi.PassivationCapable;


/**
 * A template class to all the {@link Bean} at the Eclipse JNoSQL artemis project.
 * It will work as Template method.
 *
 * @param <T>
 *            the bean type
 */
public abstract class AbstractBean<T> implements Bean<T>, PassivationCapable
{
	private final BeanManager beanManager;
	
	protected AbstractBean(final BeanManager beanManager)
	{
		this.beanManager = beanManager;
	}
	
	protected <B> B getInstance(final Class<B> clazz)
	{
		return BeanManagers.getInstance(clazz, this.beanManager);
	}
	
	protected <B> B getInstance(final Class<B> clazz, final Annotation qualifier)
	{
		return BeanManagers.getInstance(clazz, qualifier, this.beanManager);
	}
	
	@Override
	public Set<InjectionPoint> getInjectionPoints()
	{
		return Collections.emptySet();
	}
	
	@Override
	public boolean isNullable()
	{
		return false;
	}
	
	@Override
	public Class<? extends Annotation> getScope()
	{
		return ApplicationScoped.class;
	}
	
	@Override
	public String getName()
	{
		return null;
	}
	
	@Override
	public Set<Class<? extends Annotation>> getStereotypes()
	{
		return Collections.emptySet();
	}
	
	@Override
	public boolean isAlternative()
	{
		return false;
	}
	
	@Override
	public void destroy(final T instance, final CreationalContext<T> context)
	{
		// no-op by default
	}
	
}
